@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2004 by Matthew Wlazlo                                  *
- *   mwlazlo@gmail.com                                                     *
+ *   Copyright (C) 2004 by Matthew Wlazlo <mwlazlo@gmail.com>              *
+ *   Copyright (C) 2007 by Raphael Geissert <atomo64@gmail.com>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -23,6 +23,7 @@
 #define DUMP_DIR "/tmp/"
 
 #include "gmail.h"
+#include "gmailparser.h"
 #include "gmailwalletmanager.h"
 #include "prefs.h"
 #include "gmail_constants.h"
@@ -56,20 +57,19 @@ GMail::GMail() : QObject(0, "GMailNetwork")
 	isGAP4D = false;
 	
 	//Any % should be replaced with @ due to a problem with QString not looking for escaped %
-	gGMailLoginURL = "https://www.google.com/accounts/ServiceLoginAuth";
-	gGMailLoginPOSTFormat = "Email=%1&Passwd=%2&null=Sign+in&service=mail"
+	gGMailLoginURL = "https://www.google.com/accounts/ServiceLoginAuth?service=mail";
+	gGMailLoginPOSTFormat = "Email=%1&Passwd=%2&signIn=Sign+in&service=mail"
 			"&continue=http@3A@2F@2Fmail.google.com@2Fmail@3F"
-			"&ltmpl=ca_tlsosm&ltmplcache=2&rm=false&PersistentCookie=false";
+			"&ltmpl=default&ltmplcache=2&rm=false&rmShown=1";
 	gGMailCheckURL = "%1://mail.google.com/mail/?search=query"
-			"&q=@20is@3Aunread@20in@3A%2&as_subset=unread&view=tl&start=0";
+			"&q=%2&as_subset=unread&view=tl&start=0&init=1&ui=1";
 	gGMailLogOut = "https://mail.google.com/mail/?logout";
 	
 	gGAP4DLoginURL = "https://www.google.com/a/%1/LoginAction";
 	gGAP4DLoginPOSTFormat = "userName=%1&password=%2&at=null&service=mail"
-			"&continue=http@3A@2F@2Fmail.google.com@2Fa@2F%3@2F"
-			"&persistent=false";
+			"&continue=http@3A@2F@2Fmail.google.com@2Fa@2F%3";
 	gGAP4DCheckURL = "%1://mail.google.com/a/%2/?search=query"
-			"&q=@20is@3Aunread@20in@3A%3&as_subset=unread&view=tl&start=0";
+			"&q=%3&as_subset=unread&view=tl&start=0&init=1&ui=1";
 	gGAP4DLogOut = "https://mail.google.com/a/%1/?logout";
 	
 	mTimer = new QTimer(this);
@@ -101,6 +101,7 @@ void GMail::checkLoginParams()
 	
 	mUsername = username;
 	mPasswordHash = password;
+	sessionCookie = QString::null;
 	
 	useUsername = mUsername;
 	useDomain = "";
@@ -119,14 +120,12 @@ void GMail::checkLoginParams()
 	
 	kdDebug() << k_funcinfo << "Using " << useUsername << " as username and " << useDomain << " as domain" << endl;
 	
-	getURLPart(true);
-	
 	if(!mLoginLock->locked()) {
 		
 		//Try to log out if a session already exists (because it might be from an other address)
 		if(isLoggedIn(false)) {
 			kdDebug() << k_funcinfo << "A gmail session was already open, logging out from it" << endl;
-			logOut();
+			logOut(true);
 		}
 		
 		mLoginFromTimer = false;
@@ -165,9 +164,14 @@ void GMail::slotGetWalletPassword(const QString& pass)
 		LoginURL = gGMailLoginURL;
 	}
 	
-	str = QString(LoginPOSTFormat).arg(useUsername,pass).replace('@','%');
-	kdDebug() << k_funcinfo << "Sending login: " << str << endl;
+	str = QString(LoginPOSTFormat).arg(
+			KURL::encode_string(useUsername),
+			KURL::encode_string(pass)
+					  ).replace('@','%');
+	kdDebug() << k_funcinfo << "Requesting login URL" << endl;
 
+	loginRedirection = "";
+	
 	QCString b(str.utf8());
 	QByteArray postData(b);
 
@@ -176,7 +180,8 @@ void GMail::slotGetWalletPassword(const QString& pass)
 	
 	KIO::TransferJob *job = KIO::http_post(
 			LoginURL,
-	postData, false);
+			postData,
+   			false);
 	job->addMetaData("content-type", "Content-Type: application/x-www-form-urlencoded");
 	job->addMetaData("cookies", "auto");
 	job->addMetaData("cache", "reload");
@@ -186,6 +191,9 @@ void GMail::slotGetWalletPassword(const QString& pass)
 	
 	connect(job, SIGNAL(data(KIO::Job*, const QByteArray&)),
 		SLOT(slotLoginData(KIO::Job*, const QByteArray&)));
+	
+	connect(job, SIGNAL(redirection(KIO::Job*, const KURL&)),
+		SLOT(slotLoginRedirection(KIO::Job*, const KURL&)));
 }
 
 void GMail::slotLoginData(KIO::Job *job, const QByteArray &data)
@@ -193,17 +201,28 @@ void GMail::slotLoginData(KIO::Job *job, const QByteArray &data)
 	kdDebug() << k_funcinfo << endl;
 
 	if(job->error() != 0) {
-		kdDebug() << k_funcinfo << "error: " << job->errorString() << endl;
+		kdWarning() << k_funcinfo << "error: " << job->errorString() << endl;
 	} else {
 		QCString str(data, data.size() + 1);
 		mLoginBuffer.append(str);
 	}
 }
 
+void GMail::slotLoginRedirection(KIO::Job *job, const KURL &url)
+{
+	kdDebug() << k_funcinfo << url.url() << endl;
+
+	if(job->error() != 0) {
+		kdWarning() << k_funcinfo << "error: " << job->errorString() << endl;
+	} else {
+		loginRedirection = url;
+	}
+}
+
 void GMail::slotLoginResult(KIO::Job *job)
 {	
 	if(job->error() != 0) {
-		kdDebug() << k_funcinfo << "error: " << job->errorString() << endl;
+		kdWarning() << k_funcinfo << "error: " << job->errorString() << endl;
 		mLoginLock->unlock();
 		emit loginDone(false, mLoginFromTimer, job->errorString());
 	} else {
@@ -211,33 +230,68 @@ void GMail::slotLoginResult(KIO::Job *job)
 		QString redirection;
 		
 		redirection = getRedirectURL(mLoginBuffer);
+		dump2File("gmail_login.html", mLoginBuffer);
 		
 		if( redirection == QString::null ) {
 			
 			if(!isLoggedIn(false)) {
 				if(mLoginBuffer.find("onload") != -1 && (
-							mLoginBuffer.find("FixForm") != -1 ||
-							mLoginBuffer.find("start_time") != -1)) {
+						mLoginBuffer.find("FixForm") != -1 ||
+						mLoginBuffer.find("start_time") != -1)) {
 				
 					mLoginLock->unlock();
 					emit loginDone(false, mLoginFromTimer, i18n("Invalid username or password"));
+					return;
 				} else {
 					kdWarning() << k_funcinfo << " Redirection couldn't be found!" << endl;
-					dump2File("gmail_login.html",mLoginBuffer);
 					
 					mLoginLock->unlock();
 					emit loginDone(false, mLoginFromTimer, i18n("GMail's login procedure has changed, check for new version"));
+					return;
 				}
+			} else if (mLoginBuffer.find("?ui=html") != -1 && (
+				   		mLoginBuffer.find("nocheckbrowser") != -1 || 
+						mLoginBuffer.find("noscript") != -1 )) { 
+				kdDebug() << k_funcinfo << "Google is performing dirty JS check, bypassing it" << endl;
+				
+				if (loginRedirection.isEmpty()) {
+					kdWarning() << k_funcinfo << "loginRedirection is empty!" << endl;
+					mLoginLock->unlock();
+					emit loginDone(false, mLoginFromTimer, i18n("GMail's login procedure has changed, check for new version"));
+					return;
+				}
+				
+				mLoginBuffer = "";
+				
+				KURL _url;
+				
+				_url = loginRedirection;
+				_url.setQuery("ui=html&zy=n");
+				
+				if (!_url.isValid()) {
+					kdWarning() << k_funcinfo << "New _url is invalid!:" << _url.url() << endl;
+					mLoginLock->unlock();
+					// let's show a nice error message to the user instead
+					emit loginDone(false, mLoginFromTimer, i18n("GMail's login procedure has changed, check for new version"));
+					return;
+				}
+				
+				postLogin(_url.url());
+				
+				
 			} else {
 				//no more redirections?
 				kdWarning() << k_funcinfo << "No redirection was found, but seems like we are logged in!" << endl;
 				
+				mLoginBuffer = "";
 				slotPostLoginResult(job);
+				return;
 			}
 		} else {
 			mLoginBuffer = "";
 			postLogin(redirection);
 			// NOTE: LoginLock is still locked()
+			return;
 		}
 	}
 }
@@ -250,7 +304,7 @@ void GMail::postLogin(QString url)
 	// this is expected to be locked.
 	if(mLoginLock->locked()) {
 		
-		QRegExp rx("^(http[s]?://)(.*)$");
+		static QRegExp rx("^(http[s]?://)(.*)$");
 		int found;
 		
 		if(!rx.isValid()) {
@@ -261,7 +315,10 @@ void GMail::postLogin(QString url)
 		found = rx.search(url);
 		
 		if(found == -1) {
-			kdWarning() <<  "This can't be a valid url! " << url << endl;
+			kdWarning() <<  "This can't be a valid url!: " << url << endl;
+			if (!KURL(url).isValid()) {
+				kdError() <<  "This is absolutely a non-valid URL!: " << url << endl;
+			}
 		}
 		
 		if(rx.cap(1).compare("https://") != 0) {
@@ -293,7 +350,7 @@ void GMail::slotPostLoginData(KIO::Job *job, const QByteArray &data)
 	kdDebug() << k_funcinfo << endl;
 
 	if(job->error() != 0) {
-		kdDebug() << k_funcinfo << "error: " << job->errorString() << endl;
+		kdWarning() << k_funcinfo << "error: " << job->errorString() << endl;
 	} else {
 		QCString str(data, data.size() + 1);
 		mPostLoginBuffer.append(str);
@@ -303,7 +360,7 @@ void GMail::slotPostLoginData(KIO::Job *job, const QByteArray &data)
 void GMail::slotPostLoginResult(KIO::Job *job)
 {
 	if(job->error() != 0) {
-		kdDebug() << k_funcinfo << "error: " << job->errorString() << endl;
+		kdWarning() << k_funcinfo << "error: " << job->errorString() << endl;
 
 		mLoginLock->unlock();
 		emit loginDone(false, mLoginFromTimer, job->errorString());
@@ -313,12 +370,18 @@ void GMail::slotPostLoginResult(KIO::Job *job)
 		
 		if(isLoggedIn()) {
 			
+			mPostLoginBuffer = "";
 			emit loginDone(true, mLoginFromTimer);
 			checkGMail();
+			
 		} else {
+			
 			QString url = getRedirectURL(mPostLoginBuffer);
 			
 			if(url == QString::null) {
+				dump2File("gmail_postlogin.html", mPostLoginBuffer);
+				mPostLoginBuffer = "";
+				
 				emit loginDone(false, mLoginFromTimer, 
 				       i18n("Unknown error retrieving cookies"));
 			} else {
@@ -326,6 +389,7 @@ void GMail::slotPostLoginResult(KIO::Job *job)
 				mLoginLock->tryLock();
 				postLogin(url);
 			}
+			
 		}
 	}
 }
@@ -347,19 +411,17 @@ void GMail::checkGMail()
 			url = QString(gGMailCheckURL).arg(
 				(Prefs::useHTTPS()
 					? "https" 
-					: "http" ),"inbox"/*,
-				(Prefs::SearchOnlyInInbox()
-					? "inbox" 
-					: "anywhere" )*/).replace('@','%');
+					: "http" ),
+				KURL::encode_string(Prefs::searchFor())
+							 ).replace('@','%');
 		else
 			url = QString(gGAP4DCheckURL).arg(
 				(Prefs::useHTTPS()
 					? "https" 
 					: "http" ),
-				useDomain,"inbox"/*,
-				(Prefs::SearchOnlyInInbox()
-					? "inbox" 
-					: "anywhere" )*/).replace('@','%');
+				useDomain,
+				KURL::encode_string(Prefs::searchFor())
+							 ).replace('@','%');
 
 		kdDebug() << k_funcinfo << "GET: " << url << endl;
 
@@ -378,7 +440,7 @@ void GMail::checkGMail()
 void GMail::slotCheckData(KIO::Job *job, const QByteArray &data)
 {
 	if(job->error() != 0) {
-		kdDebug() << k_funcinfo << "error: " << job->errorString() << endl;
+		kdWarning() << k_funcinfo << "error: " << job->errorString() << endl;
 	} else {
 		QCString str(data, data.size() + 1);
 		mPageBuffer.append(str);
@@ -388,20 +450,26 @@ void GMail::slotCheckData(KIO::Job *job, const QByteArray &data)
 void GMail::slotCheckResult(KIO::Job *job)
 {
 	if(job->error() != 0) 
-		kdDebug() << k_funcinfo << "error: " << job->errorString() << endl;
+		kdWarning() << k_funcinfo << "error: " << job->errorString() << endl;
 
 	kdDebug() << k_funcinfo << "Check finished." << endl;
 
-	dump2File("gmail_data.html",mPageBuffer);
+	dump2File("gmail_data.html", mPageBuffer);
 	
-	static QRegExp rx("top.location=[\"|\']http[s]?://www.google.com/accounts/ServiceLogin");
+	static QRegExp rx("top\\.location=[\"\']http[s]?://www\\.google\\.com/accounts/ServiceLogin");
+	static QRegExp rx2("gmail_error=[0-9]*;");
 	int found;
 	
 	if(!rx.isValid()) {
 		kdWarning() << k_funcinfo << "Invalid RX!\n"
 				<< rx.errorString() << endl;
 	}
-			
+	
+	if(!rx2.isValid()) {
+		kdWarning() << k_funcinfo << "Invalid RX2!\n"
+				<< rx2.errorString() << endl;
+	}
+	
 	found = rx.search(mPageBuffer);
 			
 	if( found != -1 || !isLoggedIn() ) {
@@ -414,14 +482,28 @@ void GMail::slotCheckResult(KIO::Job *job)
 		mUsername = "";
 		mPasswordHash = "";
 		checkLoginParams();
-	} else {
 		
-		mTimer->start(MILLISECS(mInterval));
-		emit checkDone(mPageBuffer);
-		mPageBuffer = "";
+		return;
+	} 
+			
+	found = rx2.search(mPageBuffer);
 	
+	if( found != -1 ) {
+		kdWarning() << k_funcinfo << "Gmail is unavailable because of server-side errors!" << endl;
+		
+		mPageBuffer = "";
 		mCheckLock->unlock();
+		
+		// let's try again in 60 seconds
+		setInterval(60, true);
+		return;
 	}
+	
+	setInterval(mInterval, true);
+	emit checkDone(mPageBuffer);
+	mPageBuffer = "";
+	
+	mCheckLock->unlock();
 }
 
 ///////////////////////////////////////////////////////////////////////////
@@ -429,15 +511,13 @@ void GMail::slotCheckResult(KIO::Job *job)
 ///////////////////////////////////////////////////////////////////////////
 void GMail::slotTimeout()
 {
-	if((!mLoginLock->locked() && !isLoggedIn()) || mLoginParamsChanged) {
+	if(!isLoggedIn() || mLoginParamsChanged) {
 		mLoginFromTimer = true;
 		login();
 	} else {
-		if(isLoggedIn()) {
-			// do the check
-			mCheckFromTimer = true;
-			checkGMail();
-		}
+		// do the check
+		mCheckFromTimer = true;
+		checkGMail();
 	}
 }
 
@@ -445,10 +525,10 @@ void GMail::setInterval(unsigned int i, bool forceStart)
 {
 	bool running;
 	
-	if(i > Prefs::self()->intervalItem()->minValue().toUInt() && mInterval != i) {
+	if(i > Prefs::self()->intervalItem()->minValue().toUInt()) {
 		mInterval = i;
 		
-		if(forceStart)
+		if (forceStart)
 			running = true;
 		else
 			running = mTimer->isActive();
@@ -463,7 +543,7 @@ void GMail::setInterval(unsigned int i, bool forceStart)
 
 void GMail::setInterval(unsigned int i)
 {
-	setInterval(i,false);
+	setInterval(i, false);
 }
 
 bool GMail::isLoggedIn(bool lockCheck)
@@ -486,22 +566,17 @@ bool GMail::isLoggedIn()
 
 void GMail::slotCheckGmail()
 {
+	mCheckFromTimer = false;
 	if(!isLoggedIn()) {
-		mLoginFromTimer = false;
 		login();
 	} else {
-		mCheckFromTimer = false;
 		checkGMail();
 	}
 }
 
-QString GMail::getURLPart(bool refresh)
+QString GMail::getURLPart()
 {
-	static QString part;
-	
-	if(!refresh && part.length() > 0) {
-		return part;
-	}
+	QString part;
 	
 	if(isGAP4D) {
 		part = QString("a/%1").arg(useDomain);
@@ -512,29 +587,31 @@ QString GMail::getURLPart(bool refresh)
 	return part;
 }
 
-QString GMail::getURLPart()
+void GMail::logOut(bool force)
 {
-	return getURLPart(false);
+	if(!isLoggedIn() && !force)
+		return;
+	
+	emit logingOut();
+	
+	sessionCookie = QString::null;
+	
+	QString logoutUrl = (!isGAP4D)? gGMailLogOut : QString(gGAP4DLogOut).arg(useDomain);
+	
+	KIO::TransferJob *job = KIO::get(logoutUrl, true, false);
+	job->addMetaData("cookies", "auto");
+	job->addMetaData("cache", "reload");
+	kdDebug() << "Loging out! " << logoutUrl << endl;
 }
 
 void GMail::logOut()
 {
-	if(!isLoggedIn())
-		return;
-	
-	sessionCookie = QString::null;
-	
-	QString logout_url = (!isGAP4D)? gGMailLogOut : QString(gGAP4DLogOut).arg(useDomain);
-	
-	KIO::TransferJob *job = KIO::get(logout_url, true, false);
-	job->addMetaData("cookies", "auto");
-	job->addMetaData("cache", "reload");
-	kdDebug() << "Loging out! " << logout_url << endl;
+	logOut(false);
 }
 
 void GMail::slotLogOut()
 {
-	logOut();
+	logOut(true);
 }
 
 void GMail::dump2File(const QString filename, const QString data)
@@ -543,6 +620,8 @@ void GMail::dump2File(const QString filename, const QString data)
 	QString dump_dir = DUMP_DIR;
 		
 	dump_dir += filename;
+	
+	kdDebug() << k_funcinfo << "Dumping data to file " << dump_dir << endl;
 		
 	QFile f(dump_dir);
 		
@@ -602,7 +681,6 @@ bool GMail::cookieExists(QString cookieName,QString url)
 		kdWarning() << k_funcinfo << "Invalid RX!\n"
 				<< search.errorString() << endl;
 	}
-			
 	
 	 
 	found = search.search(cookies);
@@ -635,7 +713,7 @@ void GMail::slotSessionChanged()
 	logOut();
 	
 	if(mCheckLock->locked())
-	mCheckLock->unlock();
+		mCheckLock->unlock();
 	
 	//Clearing values will force login
 	mUsername = "";
@@ -644,28 +722,50 @@ void GMail::slotSessionChanged()
 }
 
 QString GMail::getRedirectURL(QString buffer)
-{
-	static QRegExp rx("<meta[ ]+.*url='(http[s]?://[^']+)'.*>");
+{	
+	static QRegExp metaRX("<meta[ ]+.*url='(http[s]?://[^']+)'.*>");
+	static QRegExp jsRX  ("location\\.replace[ ]*\\([ ]*['\"](http[s]?://[^'\"]+)['\"][ ]*\\)");
 	int found;
-	QString url;
+	QString url, jsurl;
 	
 	kdDebug() << k_funcinfo << endl;
 
-	if(!rx.isValid()) {
-		kdWarning() << k_funcinfo << "Invalid RX!\n"
-				<< rx.errorString() << endl;
+	if(!metaRX.isValid()) {
+		kdWarning() << k_funcinfo << "Invalid metaRX!\n"
+				<< metaRX.errorString() << endl;
 	}
-			
-	found = rx.search(buffer);
+
+	if(!jsRX.isValid()) {
+		kdWarning() << k_funcinfo << "Invalid jsRX!\n"
+				<< jsRX.errorString() << endl;
+	}
+	
+	found = metaRX.search(buffer);
 			
 	if( found == -1 ) {
 		return QString::null;
 	}
 	
-	url = KCharsets::resolveEntities(rx.cap(1));
+	url = KCharsets::resolveEntities(metaRX.cap(1));
+	
+	// now let's check if there's a JS redirection (location.replace)
+	found = jsRX.search(buffer);
+	
+	if( found == -1 ) {
+		return url;
+	}
+	jsurl = GMailParser::cleanUpData(jsRX.cap(1));
+	
+	// if both match it's ok
+	if (url.compare(jsurl) == 0) {
+		kdDebug() << k_funcinfo << "Found redirection to " << url << endl;
+		return url;
+	} else {
+		// otherwise use JS redirection
+		kdDebug() << k_funcinfo << "META and JS redirections do not match! META: " << url << " JS: " << jsurl << endl;
+		return /*js*/url;
+	}
 	
-	kdDebug() << k_funcinfo << "Found redirection to " << url << endl;
-	return url;
 }
 
 #include "gmail.moc"
