@@ -1,6 +1,7 @@
 /***************************************************************************
- *   Copyright (C) 2004 by Matthew Wlazlo                                  *
- *   mwlazlo@gmail.com                                                     *
+ *   Copyright (C) 2004 by Matthew Wlazlo <mwlazlo@gmail.com>              *
+ *   Copyright (C) 2007 by Raphael Geissert <atomo64@gmail.com>            *
+ *                                                                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -18,6 +19,7 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
+#include <cstdlib>
 #include <kaction.h>
 #include <kapplication.h>
 #include <kconfigdialog.h>
@@ -35,6 +37,8 @@
 #include <krun.h>
 #include <kiconeffect.h>
 #include <dcopclient.h>
+#include <kmimetype.h>
+#include <kurl.h>
 
 #include <qfile.h>
 #include <qpainter.h>
@@ -48,6 +52,7 @@
 #include "kcheckgmailtray.h"
 #include "loginsettingswidget.h"
 #include "netsettingswidget.h"
+#include "advancedsettingswidget.h"
 #include "prefs.h"
 
 #include "config.h"
@@ -64,7 +69,6 @@
 #define CONTEXT_NOTIFY 102
 #define CONTEXT_CHECKNOW 103
 #define CONTEXT_COMPOSE 104
-#define CONTEXT_SNOOZE 105
 
 KCheckGmailTray::KCheckGmailTray(QWidget *parent, const char *name)
 	: DCOPObject("KCheckGmailIface"),
@@ -85,7 +89,7 @@ KCheckGmailTray::KCheckGmailTray(QWidget *parent, const char *name)
 	
 	QToolTip::add(this, i18n("KCheckGMail"));
 	
-	isSnoozing = false;
+	iconDisplayed = true;
 
 	// initialise and hook up the parser
 	mParser = new GMailParser();
@@ -99,8 +103,11 @@ KCheckGmailTray::KCheckGmailTray(QWidget *parent, const char *name)
 	connect(mParser, SIGNAL(versionMismatch()), 
 		this, SLOT(slotVersionMismatch()));
 
-	connect(mParser, SIGNAL(gNameChanged(QString)), 
-		this, SLOT(slotgNameChanged(QString)));
+	connect(mParser, SIGNAL(gNameUpdate(QString)), 
+		this, SLOT(slotgNameUpdate(QString)));
+
+	connect(mParser, SIGNAL(noUnreadMail()), 
+		this, SLOT(slotNoUnreadMail()));
 	
 
 	// initialise and hook up the GMail object
@@ -147,14 +154,11 @@ KCheckGmailTray::KCheckGmailTray(QWidget *parent, const char *name)
 		i18n("&Launch Browser"), CONTEXT_LAUNCHBROWSER);
 	menu->insertItem(SmallIcon("email"),
 			 i18n("Co&mpose Mail"), CONTEXT_COMPOSE);
-	menu->insertItem(SmallIcon("clock"),
-			 i18n("&Snooze"), CONTEXT_SNOOZE);
 
 	mThreadsMenuId = menu->insertItem(SmallIcon("kcheckgmail"), i18n("Th&reads"),
 		mThreadsMenu);
 	
 	contextMenu()->setItemEnabled(mThreadsMenuId, false);
-	contextMenu()->setItemEnabled(CONTEXT_SNOOZE, false);
 
 	menu->insertSeparator();
 
@@ -177,6 +181,12 @@ KCheckGmailTray::KCheckGmailTray(QWidget *parent, const char *name)
 	kapp->dcopClient()->setDefaultObject(objId());
 }
 
+
+KCheckGmailTray::~KCheckGmailTray()
+{
+    QToolTip::remove(this);
+}
+
 void KCheckGmailTray::start()
 {
 	static bool started = false;
@@ -206,7 +216,7 @@ void KCheckGmailTray::start()
 		
 		legalCont = KMessageBox::warningContinueCancel(0,
 				i18n("Google, Gmail and Google Mail are registered trademarks of Google Inc.\n"
-						"KCheckGMail nor it's authors are in any way affiliated nor endorsed by Google Inc.\n"
+						"KCheckGMail nor its authors are in any way affiliated nor endorsed by Google Inc.\n"
 						"By using this application you may or may not be violating "
 						"the Terms of Use and/or the Program Policies "
 						"of Gmail or Google Mail.\n"
@@ -257,9 +267,6 @@ void KCheckGmailTray::slotContextMenuActivated(int n)
 		case CONTEXT_COMPOSE:
 			composeMail();
 			break;
-		case CONTEXT_SNOOZE:
-			snooze();
-			break;
 	}
 }
 
@@ -273,9 +280,12 @@ void KCheckGmailTray::launchBrowser(const QString &url)
 {
 	QString loadURL;
 
-	if(url == QString::null)
+	if(url == QString::null) {
 		loadURL = getUrlBase();
-	else
+		
+		if (Prefs::gMailSimpleInterface())
+			loadURL.append("h/");
+	} else
 		loadURL = url;
 
 	if(Prefs::useDefaultBrowser())
@@ -297,45 +307,69 @@ void KCheckGmailTray::showKNotifyDialog()
 void KCheckGmailTray::composeMail()
 {
 	QString url = getUrlBase();
-	url.append("?view=cm&fs=1&tearoff=1");
+	
+	if (Prefs::gMailSimpleInterface()) {
+		url.append("h/?v=b&pv=tl&cs=b");
+	} else {
+		url.append("?view=cm&fs=1&tearoff=1");
+	}
 	launchBrowser(url);
 }
 
-void KCheckGmailTray::snooze()
-{
-	if(isSnoozing)
-		return;
-	
-	kdDebug() << k_funcinfo << "Snoozing!" << endl;
-	
-	isSnoozing = true;
-	setPixmapSnooze();
-	contextMenu()->setItemEnabled(CONTEXT_SNOOZE, false);
-	
-	QToolTip::remove( this );
-	QToolTip::add(this, i18n("KCheckGMail - Snoozing"));
-	
-	mGmail->setInterval((Prefs::snooze()) * 60);
-}
-
 ///////////////////////////////////////////////////////////////////////////
 // Menu - related slots
 ///////////////////////////////////////////////////////////////////////////
 
 void KCheckGmailTray::slotThreadsItemHighlighted(int n)
 {
-	KNotifyClient::event(mThreadsMenu->winId(),"gmail-mail-snippet",mThreadsMenu->whatsThis(n));
+	// NOTE: the mime-type/icon code isn't enabled because KNotify won't display the icon
+	GMailParser::Thread t = mParser->getThread(n);
+	
+	if (t.subject.isEmpty()) {
+		return;
+	}
+
+	QStringList::Iterator it = t.attachments.begin();
+	QStringList attachments;
+	QString message = t.snippet,/* iconURL, format, */fileName;
+	unsigned int attachmentsCount = 0;
+	
+	/*format = i 1 8 n("format used to display the attachments (%1 is the icon, %2 is the file name)",
+		      "<img src=\"%1\"> %2");*/
+		
+	for (; it != t.attachments.end(); ++it ) {
+		attachmentsCount++;
+		fileName = *it;
+		/*iconURL = KMimeType::iconForURL(KURL(fileName));
+		kdDebug() << "Attachment name: " << fileName << ", iconURL: " << iconURL << endl;
+		attachments.append(format.arg(iconURL, fileName));*/
+		attachments.append(fileName);
+	}
+	
+	if (attachmentsCount > 0) {
+		// NOTE: %1 is the mail snippet and %2 is the attachments list
+		message = i18n("%1\nAttachment: %2", "%1\nAttachments: %2", attachmentsCount)
+				.arg(message, attachments.join(", "));
+	}
+	
+	KNotifyClient::event(mThreadsMenu->winId(), "gmail-mail-snippet", message);
 }
 
 void KCheckGmailTray::slotThreadsMenuActivated(int n)
 {
-	kdDebug() << k_funcinfo << "n=" << n << endl;
 	const GMailParser::Thread &t = mParser->getThread(n);
 
+	// make sure the thread does exist
 	if(!t.isNull) {
-		QString url=getUrlBase();
-		url.append("?view=cv&search=inbox&tearoff=1");
-		url.append("&lvp=-1&cvp=1&fs=1&tf=1&fs=1&th=");
+		QString url = getUrlBase();
+		
+		if (Prefs::gMailSimpleInterface()) {
+			url.append("h/?v=c&th=");
+		} else {
+			url.append("?view=cv&search=inbox&tearoff=1");
+			url.append("&lvp=-1&cvp=1&fs=1&tf=1&fs=1&th=");
+		}
+		
 		url.append(t.msgId);
 		launchBrowser(url);
 	}
@@ -383,6 +417,9 @@ void KCheckGmailTray::initConfigDialog()
 	AppletSettingsWidget *awid = new AppletSettingsWidget(0, "AppletSettings");
 	mConfigDialog->addPage(awid, i18n("Behavior"), "configure", i18n("Behavior"));
 
+	AdvancedSettingsWidget *cwid = new AdvancedSettingsWidget(0, "AdvancedSettings");
+	mConfigDialog->addPage(cwid, i18n("Advanced"), "package_settings", i18n("Advanced Settings"));
+
 	mLoginSettings->gmailPassword->erase();
 	mLoginSettings->gmailPassword->insert("\007\007\007");
 }
@@ -392,13 +429,14 @@ void KCheckGmailTray::slotSettingsChanged()
 	bool loginOk = true;
 	const char *passwd = mLoginSettings->gmailPassword->password();
 	const QString user = mLoginSettings->kcfg_GmailUsername->originalText();
+	int res;
 
 	kdDebug() << k_funcinfo << passwd << endl;
 	
 	if(strlen(passwd) == 0 ) {
 		kdDebug() << k_funcinfo << "user: " << user << endl;
 		if(user.length() == 0) {
-			int res = KMessageBox::warningYesNo(0, i18n("No account information has been entered. Do you want to quit?"));
+			res = KMessageBox::warningYesNo(0, i18n("No account information has been entered. Do you want to quit?"));
 
 			if( res == KMessageBox::Yes ) {
 				emit quitSelected();
@@ -408,6 +446,7 @@ void KCheckGmailTray::slotSettingsChanged()
 			}
 		}
 	} else {
+		
 		kdDebug() << k_funcinfo << " strncmp: " << strncmp(passwd, "\007\007\007", 3) << endl;
 
 		if( strncmp(passwd, "\007\007\007", 3) != 0) {
@@ -418,8 +457,43 @@ void KCheckGmailTray::slotSettingsChanged()
 		} else
 			kdDebug() << k_funcinfo << "passwd unchanged: " << passwd << endl;
 		
-		if(!isSnoozing)
-			mGmail->setInterval(Prefs::interval());
+		mGmail->setInterval(Prefs::interval());
+		
+		if (Prefs::searchFor().length() == 0) {
+			Prefs::setSearchFor("in:inbox is:unread");
+			Prefs::writeConfig();
+		}
+		
+		if (Prefs::searchFor().contains("in:") == 0 && Prefs::searchFor().contains("label:") == 0) {
+			res = KMessageBox::questionYesNo(0, 
+					i18n("<p>The search string you provided doesn't specify where to search for unread emails.</p>"
+							"<p>A search without an <i>in:</i> and <i>label:</i> will return all unread emails.</p>"
+							"<p>If what you want is to show the new emails in your inbox use <i>in:inbox</i> or leave empty.</p>"
+							"<p>Are you sure you want to proceed without specifying location?</p>"),
+       					QString::null,
+       					KStdGuiItem::yes(),
+					KStdGuiItem::no(),
+					"no_location_check");
+
+			if( res == KMessageBox::No ) {
+				QTimer::singleShot(100, this, SLOT(showPrefsDialog()));
+			}
+		}
+		if (Prefs::searchFor().contains("is:unread") == 0) {
+			res = KMessageBox::questionYesNo(0, 
+					i18n("<p>The search string you provided doesn't contain <i>is:unread</i>"
+							".</p>"
+							"<p>It should be set to ensure more unread messages are retrieved.</p>"
+							"<p>Are you sure you want to proceed?</p>"),
+					QString::null,
+					KStdGuiItem::yes(),
+					KStdGuiItem::no(),
+					"is_unread_check");
+
+			if( res == KMessageBox::No ) {
+				QTimer::singleShot(100, this, SLOT(showPrefsDialog()));
+			}
+		}
 	}
 }
 
@@ -460,9 +534,10 @@ void KCheckGmailTray::slotLoginDone(bool ok, bool evtFromTimer, const QString &w
 		
 		setPixmapAuth();
 		contextMenu()->changeItem(mCheckNowId, i18n("Login and Chec&k Mail"));
+		QToolTip::remove( this );
+		QToolTip::add(this, i18n("KCheckGMail"));
 		
 	} else {
-		contextMenu()->setItemEnabled(CONTEXT_SNOOZE, true);
 		setPixmapEmpty();
 		KNotifyClient::event(winId(), "gmail-login-yes", i18n("Now logged in to Gmail!"));
 		contextMenu()->changeItem(mCheckNowId, i18n("Chec&k Mail Now"));
@@ -472,29 +547,33 @@ void KCheckGmailTray::slotLoginDone(bool ok, bool evtFromTimer, const QString &w
 	slotMailCountChanged();
 }
 
+void KCheckGmailTray::slotLogingOut()
+{
+	// simulate a failed login attempt
+	slotLoginDone(false, true, QString::null);
+}
+
 void KCheckGmailTray::slotCheckStart()
 {
 	contextMenu()->setItemEnabled(mCheckNowId, false);
 }
 
 void KCheckGmailTray::slotCheckDone(const QString &data)
-{
-	if(isSnoozing) {
-		kdDebug() << k_funcinfo << "Finished to snooze!" << endl;
-		
-		isSnoozing = false;
-		mGmail->setInterval(Prefs::interval());
-		contextMenu()->setItemEnabled(CONTEXT_SNOOZE, true);
-		toggleAnim(true);
-		slotgNameChanged(QString::null);
-	}
-	
+{	
 	mParser->parse(data);
 	contextMenu()->setItemEnabled(mCheckNowId, true);
 }
 
 void KCheckGmailTray::slotMailArrived(unsigned int n)
 {
+	if (n == 1 && Prefs::displaySubjectOnSingleMail()) {
+		GMailParser::Thread t;
+		t = mParser->getLastThread();
+		if (!t.isNull) {
+			slotMailArrived(t.subject);
+			return;
+		}
+	}
 	QString str;
 
 	str = i18n("There is <b>1</b> new message",
@@ -504,6 +583,21 @@ void KCheckGmailTray::slotMailArrived(unsigned int n)
 	slotMailCountChanged();
 }
 
+void KCheckGmailTray::slotMailArrived(QString subject)
+{
+	QString str;
+	
+	str = i18n("New mail arrived: <i>%1</i>").arg(subject);
+	
+	KNotifyClient::event(winId(), "new-gmail-arrived", str);
+	slotMailCountChanged();
+}
+
+void KCheckGmailTray::slotNoUnreadMail()
+{
+	KNotifyClient::event(winId(), "no-unread-gmail", i18n("There are no unread messages"));
+}
+
 void KCheckGmailTray::slotMailCountChanged()
 {
 	mMailCount = mParser->getNewCount(true);
@@ -536,7 +630,6 @@ void KCheckGmailTray::updateThreadMenu()
 					str.replace("&","&&");
 					
 					id = mThreadsMenu->insertItem(str, t.id);
-					mThreadsMenu->setWhatsThis(id, t.snippet);
 					numItems ++;
 				}
 			}
@@ -557,6 +650,98 @@ void KCheckGmailTray::slotSessionChanged()
 ///////////////////////////////////////////////////////////////////////////
 
 //Used by the DCOP interface
+void KCheckGmailTray::showIcon()
+{
+	iconDisplayed = true;
+	show();
+}
+
+void KCheckGmailTray::hideIcon()
+{
+	iconDisplayed = false;
+	hide();
+}
+
+QStringList KCheckGmailTray::getThreads()
+{
+	QStringList out;
+	QMap<QString,bool> *threads = mParser->getThreadList();
+
+	if(threads) {
+
+		QValueList<QString> klist = threads->keys();
+		QValueList<QString>::iterator iter = klist.begin();
+		
+		while(iter != klist.end()) {
+			const GMailParser::Thread &t = mParser->getThread(*iter);
+			if(!t.isNull) {
+				out.append(*iter);
+			}
+			iter ++;
+		}
+	}
+	return out;
+}
+
+QString KCheckGmailTray::getThreadSubject(QString msgId)
+{
+	GMailParser::Thread t = mParser->getThread(msgId);
+	
+	if (t.subject.isEmpty()) {
+		return QString::null;
+	}
+	
+	return t.subject;
+}
+
+QString KCheckGmailTray::getThreadSender(QString msgId)
+{
+	GMailParser::Thread t = mParser->getThread(msgId);
+	
+	if (t.senders.isEmpty()) {
+		return QString::null;
+	}
+	
+	return t.senders;
+}
+
+QString KCheckGmailTray::getThreadSnippet(QString msgId)
+{
+	GMailParser::Thread t = mParser->getThread(msgId);
+	
+	if (t.snippet.isEmpty()) {
+		return QString::null;
+	}
+	
+	return t.snippet;
+}
+
+QStringList KCheckGmailTray::getThreadAttachments(QString msgId)
+{
+	GMailParser::Thread t = mParser->getThread(msgId);
+	
+	return t.attachments;
+}
+
+bool KCheckGmailTray::isNewThread(QString msgId)
+{
+	GMailParser::Thread t = mParser->getThread(msgId);
+	
+	return t.isNew;
+}
+
+QMap<QString, unsigned int> KCheckGmailTray::getLabels()
+{
+	QMap<QString, unsigned int> labels = mParser->getLabels();
+	
+	return labels;
+}
+
+QString KCheckGmailTray::getGaiaName()
+{
+	return mParser->getGaiaName();
+}
+
 void KCheckGmailTray::checkMailNow()
 {
 	mGmail->slotCheckGmail();
@@ -600,10 +785,6 @@ void KCheckGmailTray::updateCountImage()
 		QFontMetrics qfm(countFont);
 		int width = qfm.width(countString);
 
-		kdDebug() << "------- countFontSize=" << countFontSize 
-				<< " width=" << width << " w=" << w << endl;
-
-		kdDebug() << "pixelSize="<<countFont.pixelSize()<<endl;
 		if(width > w) {
 			countFontSize *= float(w) / float(width);
 			countFont.setPointSizeFloat( countFontSize );
@@ -629,7 +810,11 @@ void KCheckGmailTray::updateCountImage()
 //from rsibreak: rsiwidget.cpp
 void KCheckGmailTray::whereAmI()
 {
+	if (!iconDisplayed)
+		showIcon();
+	
 	takeScreenshotOfTrayIcon();
+	
 	KMessageBox::information(0,
 				 i18n("<p>KCheckGMail is already running</p><p>You can find it here:</p><p><center><img source=\"systray_shot\"></center></p>"),
 				 i18n("Already Running"));
@@ -638,6 +823,7 @@ void KCheckGmailTray::whereAmI()
 //from rsibreak: rsiwidget.cpp
 void KCheckGmailTray::takeScreenshotOfTrayIcon()
 {
+	
         // Process the events else the icon will not be there and the screenie will fail!
 	kapp->processEvents();
 
@@ -646,13 +832,24 @@ void KCheckGmailTray::takeScreenshotOfTrayIcon()
         // The part about the border is copied from  KSystemTray::displayCloseMessage()
 	//
         // Compute size and position of the pixmap to be grabbed:
-	QPoint g = this->mapToGlobal( this-> pos() );
+	QPoint g = this-> pos();
+
+        //Catch invalid positions (2007 - Raphael Geissert)
+        if (g.x() < 0) {
+            g.setX(0);
+        }
+        if (g.y() < 0) {
+            g.setY(0);
+        }
+        g = this->mapToGlobal( g );
+
 	int desktopWidth  = kapp->desktop()->width();
 	int desktopHeight = kapp->desktop()->height();
 	int tw = this->width();
 	int th = this->height();
 	int w = desktopWidth / 4;
 	int h = desktopHeight / 9;
+	
 	int x = g.x() + tw/2 - w/2;               // Center the rectange in the systray icon
 	int y = g.y() + th/2 - h/2;
 	if ( x < 0 )                 x = 0;       // Move the rectangle to stay in the desktop limits
@@ -686,7 +883,7 @@ void KCheckGmailTray::takeScreenshotOfTrayIcon()
 }
 
 
-void KCheckGmailTray::slotgNameChanged(QString name)
+void KCheckGmailTray::slotgNameUpdate(QString name)
 {
 	static QString sname;
 	kdDebug() << k_funcinfo << "Updating tooltip" << endl;
@@ -694,6 +891,8 @@ void KCheckGmailTray::slotgNameChanged(QString name)
 	//Trick to restore the tooltip
 	if(name == QString::null)
 		name = sname;
+	else
+		sname = name;
 	
 	QToolTip::remove( this );
 	QToolTip::add(this, i18n("KCheckGMail - Notifying about new email for %1").arg(name));
@@ -703,16 +902,7 @@ void KCheckGmailTray::setPixmapAuth()
 {
 	setPixmap(mIconEffect.apply(mPixGmail, 
 		  KIconEffect::Colorize,
-		  0.70,
-		  Qt::red,
-		  false));
-}
-
-void KCheckGmailTray::setPixmapSnooze()
-{
-	setPixmap(mIconEffect.apply(mPixCount, 
-		  KIconEffect::Colorize,
-		  0.75,
+		  0.60,
 		  Qt::lightGray,
 		  false));
 }
